#ifndef CONFIG_H
#define CONFIG_H

#include "drivers/unix_fs/unix_fs_driver.h"
#include "driver.h"

#include <sys/time.h>
#include <libconfig.h>

enum ConfigPreloadError {
    CFG_PLD_INVALID = -2,
    CFG_PLD_NOT_FOUND,
    CFG_PLD_OK
};

static char *config_preload_paths[] = {"discuss.conf", "/etc/discuss.conf", NULL};
extern config_t config;

/* first tries to load config `filename`, then tries preload_paths in order
 * on first invalid config returns CFG_PLD_INVALID,
 * if all paths do not exist returns CFG_PLD_NOT_FOUND
 * if filename is NULL, checks only preload_paths
 */
int preload_config(const char *filename);

#pragma GCC diagnostic ignored "-Wincompatible-pointer-types"
#define CONF_ROOT_LOOKUP(type, name) config_lookup_##type(&config, #name, &name)

/* defaults */
static int comments_per_page = 20;

static int max_name_size = 1024;
static int max_comment_size = 4096;

static int user_max_length = 64;
static int password_max_length = 256;

static int session_expire = 86400;

static char *qments_path = "qments-storage";
static UnixFsDriverData driver_data;
static Driver driver = unix_fs_driver;

static char *session_cookie_name = "thecookie";
static char *hostname = "localhost";


#endif /* CONFIG_H */
