#include "cgic.h"
#include "auth.h"
#include "config.h"
#include "utils.h"

#include <string.h>
#include <syslog.h>
#include <stdlib.h>

#define UUID_SIZE 37

/* TODO: place cookie normally */
char *
validate_credentials()
{
    char username[user_max_length + 1], password[password_max_length + 1];
    char session_id[UUID_SIZE];
    cgiFormResultType err;
    int auth;

    err = cgiFormString("username", username, user_max_length + 1);
    if (err == cgiFormTruncated) {
        return "Username too long\n";
    }
    if (err == cgiFormNotFound) {
        return "Username not provided\n";
    }

    err = cgiFormString("password", password, password_max_length + 1);
    if (err == cgiFormTruncated) {
        return "Password too long\n";
    }
    if (err == cgiFormNotFound) {
        return "Password not provided\n";
    }

    if (!is_valid_username(username)) {
        return "Username must be [A-Za-z0-9_]\n";
    }
    
    if (!is_valid_password(password)) {
        return "Password must be a sequence of bytes in range 32-255\n";
    }

    auth = authenticate(username, password, session_id);
    if (auth < 0) {
        return "Some error occured, contact system administrator\n";
    }

    if (auth) {
        cgiHeaderCookieSet(session_cookie_name, session_id, session_expire, "/", hostname, 0);
        return "You've successfully logged in!\n";
    } else {
        return "Failed to log in, check credentials\n";
    }
}

void
print_login_form()
{
    fputs("<form action=\"\" method=\"POST\">\n"
        "<label>Username:</label>\n"
        "<input type=\"text\" name=\"username\"><br>\n"
        "<label>Password: </label>\n"
        "<input type=\"password\" name=\"password\"><br>\n"
        "<input type=\"submit\" name=\"login\" value=\"Submit\">\n"
    "</form>\n", cgiOut);
}

int
cgiMain()
{
    int cfg_pld_err;
    char *message;
    
    config_init(&config);
    cfg_pld_err = preload_config((cgiArgc > 1 ? cgiArgv[1] : NULL));

    if (cfg_pld_err == CFG_PLD_INVALID) {
        syslog(LOG_ERR, "Got invalid config\n");
        syslog(LOG_ERR, "%s\n", config_error_text(&config));
        exit(EXIT_FAILURE);
    } else if (cfg_pld_err == CFG_PLD_NOT_FOUND) {
        syslog(LOG_WARNING, "Config not found, loading defaults\n");
    } else {
        load_auth_config();
    }

    CONF_ROOT_LOOKUP(int, session_expire);

    if (cgiFormSubmitClicked("login") == cgiFormSuccess &&
            !strcmp(cgiRequestMethod, "POST")) {
        message = validate_credentials();
    } else {
        message = "";
    }

    cgiHeaderContentType("text/html; charset=utf-8");

    fprintf(cgiOut, "<html>\n");

    fprintf(cgiOut, "<head>\n");
    fprintf(cgiOut, "<title> Simple discuss powered by qments </title>\n");
    fprintf(cgiOut, "</head>\n");

    fprintf(cgiOut, "<body>\n");

    fputs(message, cgiOut);

    print_login_form();

    fprintf(cgiOut, "</body>\n");

    fprintf(cgiOut, "</html>\n");

    return 0;

}
